rm(list=ls()) ; graphics.off() ; gc()

#Load libraries
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)
library(fields)
library(metR)
#library(transformeR)
#library(loadeR)
#library(climate4R.value)
#library(visualizeR)
#library(trend)
library(plotrix)
#library(miceadds)

rm(list=ls()) ; gc()

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/output/D4/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

avgmean <- function(x,ventana){
  value <- rep(NA,length(x))
  for(i in (1+ventana):(length(x)-ventana)){
    value[i] <- mean(x[(i-ventana):(i+ventana)],na.rm=T)
  }
  return(value)
}
####################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_50km_1x1.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

# domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain
# 
# domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean
# 
# domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe
# 
# domain <- domain2 

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
var <- scale(t(var),center=T,scale=T)
rm(nc_var) ; gc()

#Load clustering information
do <- "D4"
dom <- do[1]
k=8 

#Preparing data
longitude=rep(nc_lon,length(nc_lat))
latitude=rep(nc_lat,each=length(nc_lon))

Z <- list() 

load(file = paste("WardCluster_Dendrogram_","D4","_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)

# for(cp in 1:k){
#   Z[[cp]] <-colMeans(var[which(vector_cluster==cp),],na.rm=T)
# }
# 
# datitosERA5 <- data.frame(Longitude=rep(longitude,k),
#                           Latitude=rep(latitude,k),
#                           Z = unlist(Z),
#                           CP = rep(1:k,each=length(unlist(Z))/k))
# datitosERA5$CP <- factor(datitosERA5$CP,levels=c(1:k))

#Temporal cycle
calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
n_years <- length(fechas)/365

datos = vector_cluster
if(length(datos)!=length(fechas)){
  datos <- datos[which(calendario%in%fechas)]
  rm(calendario,fechas)
}

datos2 = data.frame(group=as.numeric( datos),
                    day=rep(1:365,n_years))

valueERA5 <- matrix(NA,365,k)
for(i in 1:365){
  for(j in 1:k){
    valueERA5[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
  }
}

# avgERA5 <- matrix(NA,365,k)
# for(j in 1:k){
#   avgERA5[,j] <- avgmean(x=valueERA5[,j],ventana = 15)
# }

################################################################################
#Start working with models
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/esmvaltool_output/recipe_intvar_20240523_225001/preproc/diagnostic_2/psl_d1/")
errors <- list()
gcms <- list()
runs <- list()

for(f in 1:length(files)){
  nc2 <- nc_open(paste("/esarchive/scratch/molmo/esmvaltool_output/recipe_intvar_20240523_225001/preproc/diagnostic_2/psl_d1/",files[f],sep=""))
  run <- ncatt_get(nc2,0,)$variant_label
  gcm <- ncatt_get(nc2,0,)$source_id
  
  load(file = paste("/esarchive/scratch/molmo/output/D4/WardClustering_D4.8_D4_1x1",gcm,"_",run,".Rdata",sep=""))
  
  nc_lon <- ncvar_get(nc2,"lon")
  nc_lat <- ncvar_get(nc2,"lat")
  nc_var <- ncvar_get(nc2,"psl")
  var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])
  
  OUTPUTS <- "/esarchive/scratch/molmo/output/D4/intvar/"
  
  message("Estimating ",gcm," ",run)
  vector_cluster <- cutree(ward_clustering,k)
  
  load(paste(OUTPUTS,"orden_CPs_",dom,"_",gcm,"_",run,"_",k,".Rdata",sep=""))
  
  #Reordenamiento
  datos <- vector_cluster
  for(i in 1:k){
    datos[which(vector_cluster==i)] <- orden[i]
  }
  
  #Temporal cycle
  calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  n_years <- length(fechas)/365
  
  if(length(datos)!=length(fechas)){
    datos <- datos[which(calendario%in%fechas)]
    rm(calendario,fechas)
  }
  
  datos2 = data.frame(group=as.numeric( datos),
                      day=rep(1:365,n_years))
  
  value <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      value[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
    }
  }
  
  ErrorGCM <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      ErrorGCM[i,j] <- abs(valueERA5[i,j] - value[i,j])#/valueERA5[k,j]*100
    }
  }
  
  errors[[f]] <- round(colMeans(ErrorGCM,na.rm = T),digits=1)
  gcms[[f]] <- gcm
  runs[[f]] <- run
  
}
  name <- list()
  for(f in 1:length(gcms)){
    name[[f]] <- paste(gcms[[f]],runs[[f]],sep=" ")
  }
  
  #Heatmaps ErrorCycle
  data = data.frame(Error=unlist(errors),GCM=rep(unlist(gcms),each=k),Run=rep(unlist(runs),each=k),GCM_Run=rep(unlist(name),each=k),CP=rep(c(1:k),length(unlist(gcms))))
  
  data.mean <- data.frame(Error=as.numeric(lapply(X=errors,FUN=mean)),GCM=rep(unlist(gcms),each=k),Run=rep(unlist(runs),each=k),GCM_Run=unlist(name),CP=rep("Mean",length(unlist(gcms))))
  data.mean$Error <- round(data.mean$Error,digits=1)
  
  #data$Error[which(data$Error > 20)] <- 20
  data$GCM_Run <- factor(data$GCM_Run,levels=rev(unique(data$GCM_Run)))
  data <- rbind(data,data.mean)
  
  orden.run <- NA
  for(r in 1:50){
    orden.run[r] <- paste("r",r,"i1p1f1",sep="")
  }
  
  data$Run <- factor(data$Run,levels=rev(orden.run))
  data <- data[-which(data$GCM=="CanESM5"),]
  #data <- data[-which(data$GCM%in%c("NESM3","KACE-1-0-G")),]
  p <- ggplot(data=data,aes(x=CP,y=Run,fill=Error)) + geom_tile() + coord_cartesian(expand=F) + theme_bw()+
    scale_fill_gradientn(name="days",colours=rev(brewer.pal(11,"Spectral")),limits=c(0,20)) + facet_wrap(~GCM,nrow = 1) #+geom_text(aes(label=Error))
  ggsave(p,filename=paste(OUTPUTS,"ErrorCycle_GCMs_IntVar",dom,"_","CPs.png",sep=""),device="png",
         dpi=600,width=9,height = 7)  
  
  save(data,file="/esarchive/scratch/molmo/output/D4/ErrorCycle_D4.Rdata")
  
  #Order (lowest to highest Error values)
  ranking <- cbind(data.mean$GCM[order(data.mean$Error)],data.mean$Error[order(data.mean$Error)])
  